package Data::Sah::Type::code;

use Moo::Role;
with 'Data::Sah::Type::BaseType';

our $VERSION = '0.08'; # VERSION

1;
# ABSTRACT: code type


__END__
=pod

=head1 NAME

Data::Sah::Type::code - code type

=head1 VERSION

version 0.08

=head1 AUTHOR

Steven Haryanto <stevenharyanto@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Steven Haryanto.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

